@@ -27,6 +27,9 @@ quick_error! {
             from()
             display("Error in lua: {}", err)
         }
+        Clipboard {
+            display("Error using clipboard")
+        }
         None
     }
 }
